@@ -26,42 +26,85 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strings.h>
-#include <termios.h>
 #include <time.h>
 #include <unistd.h>
+#include <sys/types.h>
 
 #include "common.h"
+#include "stoken.h"
 #include "securid.h"
 #include "sdtid.h"
 #include "stoken-internal.h"
 
-static void print_token_info_line(const char *key, const char *value)
-{
-	/* require --seed to show anything sensitive */
-	if (strcasestr(key, "seed") && !opt_seed)
-		return;
-	printf("%-24s: %s\n", key, value);
-}
+#ifdef _WIN32
 
-static int raw_read_user_input(char *out, int max_len)
+static int plat_read_user_input(char *out, int max_len, int hide_chars)
 {
+	/* TODO: Hide passwords */
 	char *p;
 
+	fgets(out, max_len, stdin);
+	p = strchr(out, '\n');
+	if (p)
+		*p = 0;
+	return 0;
+}
+
+static void terminal_init(void)
+{
+}
+
+static int fork_and_wait(void)
+{
+	/* TODO */
+	die("Subprocess support is not yet implemented on Windows.\n");
+	return -EINVAL;
+}
+
+#else /* _WIN32 */
+
+#include <termios.h>
+#include <sys/wait.h>
+
+static struct termios oldtio;
+
+static void stdin_echo(int enable_echo)
+{
+	struct termios tio = oldtio;
+	const int fd = 0;
+
+	if (!enable_echo) {
+		/* ripped from busybox bb_ask() */
+		tcflush(fd, TCIFLUSH);
+		tio.c_lflag &= ~(ECHO|ECHOE|ECHOK|ECHONL);
+		tcsetattr(fd, TCSANOW, &tio);
+	} else
+		tcsetattr(fd, TCSANOW, &oldtio);
+}
+
+static int plat_read_user_input(char *out, int max_len, int hide_chars)
+{
+	char *p;
+	int ret = 0;
+
+	stdin_echo(!hide_chars);
 	fflush(stdout);
 	fflush(stderr);
 	if (fgets(out, max_len, stdin) == NULL) {
 		*out = 0;
-		return 0;
+		goto done;
 	}
-	p = index(out, '\n');
+	p = strchr(out, '\n');
 	if (p)
 		*p = 0;
-	return strlen(out);
-}
+	ret = strlen(out);
 
-static struct termios oldtio;
-static void stdin_echo(int enable_echo);
+done:
+	stdin_echo(1);
+	if (hide_chars)
+		puts("");
+	return ret;
+}
 
 static void restore_tio(int sig)
 {
@@ -85,24 +128,28 @@ static void terminal_init(void)
 	tcgetattr(fd, &oldtio);
 }
 
-static void stdin_echo(int enable_echo)
+static int fork_and_wait(void)
 {
-	struct termios tio = oldtio;
-	const int fd = 0;
+	pid_t child = fork();
 
-	if (!enable_echo) {
-		/* ripped from busybox bb_ask() */
-		tcflush(fd, TCIFLUSH);
-		tio.c_lflag &= ~(ECHO|ECHOE|ECHOK|ECHONL);
-		tcsetattr(fd, TCSANOW, &tio);
-	} else
-		tcsetattr(fd, TCSANOW, &oldtio);
+	if (child < 0)
+		die("can't fork\n");
+	else if (child == 0)
+		return 0;
+	else if (child > 0) {
+		int rv;
+		wait(&rv);
+		if (!WIFEXITED(rv) || WEXITSTATUS(rv))
+			exit(1);
+	}
+	return 1;
 }
 
+#endif /* _WIN32 */
+
 static int read_user_input(char *out, int max_len, int hide_chars)
 {
 	static int first = 1;
-	int rc;
 
 	if (opt_stdin) {
 		if (!first) {
@@ -110,7 +157,7 @@ static int read_user_input(char *out, int max_len, int hide_chars)
 			die("error: --stdin only allows one prompt\n");
 		}
 		first = 0;
-		return raw_read_user_input(out, max_len);
+		return plat_read_user_input(out, max_len, hide_chars);
 	}
 
 	if (opt_batch) {
@@ -118,20 +165,27 @@ static int read_user_input(char *out, int max_len, int hide_chars)
 		die("error: --batch mode specified but command-line input is requested\n");
 	}
 
-	stdin_echo(!hide_chars);
-	rc = raw_read_user_input(out, max_len);
-	stdin_echo(1);
-
-	if (hide_chars)
-		puts("");
-	return rc;
+	return plat_read_user_input(out, max_len, hide_chars);
 }
 
-static time_t adjusted_time(void)
+static void print_token_info_line(const char *key, const char *value)
+{
+	/* require --seed to show anything sensitive */
+	if (strcasestr(key, "seed") && !opt_seed)
+		return;
+	printf("%-24s: %s\n", key, value);
+}
+
+static time_t adjusted_time(struct securid_token *t)
 {
 	time_t now = time(NULL);
 	long new_time;
 
+	if (opt_next && opt_use_time)
+		die("error: --use-time and --next are mutually exclusive\n");
+	if (opt_next)
+		return now + securid_token_interval(t);
+
 	if (!opt_use_time)
 		return now;
 	else if (sscanf(opt_use_time, "+%ld", &new_time) == 1)
@@ -147,24 +201,33 @@ static time_t adjusted_time(void)
 
 static void request_devid(struct securid_token *t, char *devid)
 {
-	int i, rc;
+	int i;
 
 	if (opt_devid) {
-		rc = securid_decrypt_seed(t, "", opt_devid);
-		if (rc != ERR_BAD_DEVID) {
+		if (securid_check_devid(t, opt_devid) == ERR_NONE) {
 			xstrncpy(devid, opt_devid, BUFLEN);
 			return;
 		}
 		warn("warning: --devid parameter is incorrect\n");
+	} else {
+		const struct stoken_guid *glist = stoken_get_guid_list();
+		for (i = 0; glist[i].tag != NULL; i++) {
+			if (securid_check_devid(t, glist[i].guid) == ERR_NONE) {
+				prompt("Using class GUID for %s; use --devid to override\n",
+				       glist[i].long_name);
+				strncpy(devid, glist[i].guid, BUFLEN);
+				return;
+			}
+		}
 	}
 
 	prompt("This token is bound to a specific device.\n");
 	for (i = 0; ; i++) {
 		prompt("Enter device ID from the RSA 'About' screen: ");
-		read_user_input(devid, BUFLEN, 0);
+		if (read_user_input(devid, BUFLEN, 0) == 0)
+			continue;
 
-		rc = securid_decrypt_seed(t, "", devid);
-		if (rc != ERR_BAD_DEVID)
+		if (securid_check_devid(t, devid) == ERR_NONE)
 			return;
 		if (i == 2)
 			die("error: invalid device ID\n");
@@ -189,7 +252,8 @@ static void request_pass(const char *prompt_msg, struct securid_token *t,
 
 	for (i = 0; ; i++) {
 		prompt(prompt_msg);
-		read_user_input(pass, BUFLEN, 1);
+		if (read_user_input(pass, BUFLEN, 1) == 0)
+			continue;
 
 		rc = securid_decrypt_seed(t, pass, devid);
 		if (rc == ERR_DECRYPT_FAILED) {
@@ -300,6 +364,79 @@ static void print_formatted(const char *buf)
 	free(formatted);
 }
 
+static void display_qr(const char *filename)
+{
+	const char *programs[] = {
+		/*
+		 * I'd like to include xdg-open here, but it insists on
+		 * opening the file in the background, which races with the
+		 * temporary file cleanup.
+		 */
+		"display",	/* ImageMagick */
+		"eog",		/* Eye of GNOME */
+		"gwenview",	/* KDE viewer */
+		"ristretto",	/* Xfce */
+		NULL,
+	};
+	const char **p, *user;
+
+	if (fork_and_wait() != 0)
+		return;
+
+	user = getenv("QR_VIEWER");
+	if (user) {
+		execlp(user, user, filename, NULL);
+		die("unable to execute '%s'\n", user);
+	}
+
+	for (p = programs; *p; p++)
+		execlp(*p, *p, filename, NULL);
+
+	die("can't find a suitable image viewer; try setting $QR_VIEWER\n");
+}
+
+static void __export_qr(const char *filename, const char *token)
+{
+	if (fork_and_wait() != 0)
+		return;
+	execlp("qrencode", "qrencode", "-l", "H", "-o", filename,
+	       token, NULL);
+	die("can't exec qrencode (is it in your PATH?)\n");
+}
+
+static void export_qr(const char *filename, const char *token)
+{
+	char *formatted;
+
+	if (opt_blocks) {
+		warn("warning: --blocks is invalid in QR mode; using --android\n");
+		opt_android = 1;
+		opt_blocks = 0;
+	}
+
+	if (!(opt_android || opt_iphone || opt_v3))
+		opt_android = 1;
+
+	formatted = format_token(token);
+
+	if (filename)
+		__export_qr(filename, formatted);
+	else {
+		char fname[64];
+		int fd;
+
+		snprintf(fname, sizeof(fname), "%s/XXXXXX.png",
+			 getenv("TMPDIR") ? : "/tmp");
+		fd = mkstemps(fname, 4);
+		if (fd < 0)
+			die("can't create temp file '%s'\n", fname);
+		__export_qr(fname, formatted);
+		display_qr(fname);
+		unlink(fname);
+	}
+	free(formatted);
+}
+
 int main(int argc, char **argv)
 {
 	char *cmd = parse_cmdline(argc, argv, NOT_GUI);
@@ -326,13 +463,15 @@ int main(int argc, char **argv)
 	terminal_init();
 
 	if (!strcmp(cmd, "tokencode")) {
-		int days_left = securid_check_exp(t, adjusted_time());
+		int days_left;
 
+		unlock_token(t, 1, NULL);
+
+		days_left = securid_check_exp(t, adjusted_time(t));
 		if (days_left < 0 && !opt_force)
 			die("error: token has expired; use --force to override\n");
 
-		unlock_token(t, 1, NULL);
-		securid_compute_tokencode(t, adjusted_time(), buf);
+		securid_compute_tokencode(t, adjusted_time(t), buf);
 		puts(buf);
 
 		if (days_left < 14 && !opt_force)
@@ -348,8 +487,7 @@ int main(int argc, char **argv)
 		}
 
 		t->is_smartphone = 0;
-		t->version = 2;
-		securid_encode_token(t, pass, opt_new_devid, buf);
+		securid_encode_token(t, pass, opt_new_devid, 2, buf);
 		rc = write_token_and_pin(buf, NULL, pass);
 		if (rc != ERR_NONE)
 			die("rcfile: error writing new token: %s\n",
@@ -364,10 +502,15 @@ int main(int argc, char **argv)
 			pass = NULL;
 
 		if (!opt_sdtid) {
-			t->is_smartphone = opt_iphone || opt_android || opt_v3;
-			t->version = opt_v3 ? 3 : 2;
-			securid_encode_token(t, pass, opt_new_devid, buf);
-			print_formatted(buf);
+			t->is_smartphone = opt_iphone || opt_android ||
+					   opt_v3 || opt_show_qr || opt_qr;
+			securid_encode_token(t, pass, opt_new_devid,
+					     opt_v3 ? 3 : 2, buf);
+
+			if (opt_show_qr || opt_qr)
+				export_qr(opt_show_qr ? NULL : opt_qr, buf);
+			else
+				print_formatted(buf);
 		} else {
 			rc = sdtid_export(opt_template, t, pass, opt_new_devid);
 			if (rc != ERR_NONE)
@@ -403,7 +546,7 @@ int main(int argc, char **argv)
 				die("error: PINs do not match\n");
 		}
 
-		securid_encode_token(t, pass, NULL, buf);
+		securid_encode_token(t, pass, NULL, 2, buf);
 		rc = write_token_and_pin(buf, len ? pin : NULL, pass);
 		free(pass);
 
@@ -414,7 +557,7 @@ int main(int argc, char **argv)
 
 		unlock_token(t, 0, NULL);
 		request_new_pass(pass);
-		securid_encode_token(t, pass, NULL, buf);
+		securid_encode_token(t, pass, NULL, 2, buf);
 
 		/* just print to stdout if it didn't come from the rcfile */
 		if (opt_file || opt_token)
