@@ -26,10 +26,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#ifdef HAVE_MLOCKALL
+#include <sys/mman.h>
+#endif
+
 #include "common.h"
 #include "securid.h"
 #include "stoken.h"
@@ -38,11 +41,11 @@
 /* globals - shared with cli.c or gui.c */
 
 int opt_random, opt_keep_password, opt_blocks, opt_iphone, opt_android,
-	opt_v3, opt_seed, opt_sdtid, opt_small;
+	opt_v3, opt_show_qr, opt_seed, opt_sdtid, opt_small, opt_next;
 int opt_debug, opt_version, opt_help, opt_batch, opt_force, opt_stdin;
 char *opt_rcfile, *opt_file, *opt_token, *opt_devid, *opt_password,
      *opt_pin, *opt_use_time, *opt_new_password, *opt_new_devid,
-     *opt_new_pin, *opt_template;
+     *opt_new_pin, *opt_template, *opt_qr;
 struct securid_token *current_token;
 
 static int debug_level;
@@ -133,6 +136,7 @@ enum {
 	OPT_NEW_DEVID,
 	OPT_NEW_PIN,
 	OPT_TEMPLATE,
+	OPT_QR,
 };
 
 static const struct option long_opts[] = {
@@ -153,16 +157,19 @@ static const struct option long_opts[] = {
 	/* global: misc/debug */
 	{ "debug",          0, NULL,                    'd'               },
 	{ "version",        0, NULL,                    'v'               },
+	{ "force",          0, NULL,                    'f'               },
+	{ "use-time",       1, NULL,                    OPT_USE_TIME      },
 	{ "help",           0, NULL,                    'h'               },
 
 	/* all remaining options are for CLI only */
 #define FINAL_GUI_OPTION	"help"
 
 	{ "batch",          0, NULL,                    'b'               },
-	{ "force",          0, NULL,                    'f'               },
-	{ "use-time",       1, NULL,                    OPT_USE_TIME      },
 
-	/* used for specific commands */
+	/* used for tokencode generation */
+	{ "next",           0, &opt_next,               1                 },
+
+	/* these are mostly for exporting/issuing tokens */
 	{ "new-password",   1, NULL,                    OPT_NEW_PASSWORD  },
 	{ "new-devid",      1, NULL,                    OPT_NEW_DEVID     },
 	{ "new-pin",        1, NULL,                    OPT_NEW_PIN       },
@@ -174,6 +181,8 @@ static const struct option long_opts[] = {
 	{ "v3",             0, &opt_v3,                 1                 },
 	{ "sdtid",          0, &opt_sdtid,              1                 },
 	{ "xml",            0, &opt_sdtid,              1                 },
+	{ "qr",             1, NULL,                    OPT_QR            },
+	{ "show-qr",        0, &opt_show_qr,            1                 },
 	{ "seed",           0, &opt_seed,               1                 },
 	{ "stdin",          0, NULL,                    's'               },
 	{ NULL,             0, NULL,                    0                 },
@@ -213,7 +222,8 @@ static void usage_cli(void)
 	puts("Other commands:");
 	puts("");
 	puts("  stoken show [ --seed ]");
-	puts("  stoken export [ { --blocks | --iphone | --android | --v3 | --sdtid } ]");
+	puts("  stoken export [ { --blocks | --iphone | --android | --v3 | --sdtid |");
+	puts("                    --qr=<file> | --show-qr } ]");
 	puts("  stoken issue [ --template=<sdtid_skeleton> ]");
 	puts("");
 	usage_common();
@@ -267,6 +277,7 @@ char *parse_cmdline(int argc, char **argv, int is_gui)
 		case OPT_NEW_DEVID: opt_new_devid = optarg; break;
 		case OPT_NEW_PIN: opt_new_pin = optarg; break;
 		case OPT_TEMPLATE: opt_template = optarg; break;
+		case OPT_QR: opt_qr = optarg; break;
 		case 0: break;
 		default: opt_help = 1;
 		}
@@ -322,7 +333,7 @@ static int read_token_from_file(char *filename, struct securid_token *t)
 		if (rc != ERR_GENERAL)
 			break;
 
-		p = index(p, '\n');
+		p = strchr(p, '\n');
 		if (!p)
 			break;
 		p++;
@@ -367,6 +378,8 @@ int common_init(char *cmd)
 	mlockall(MCL_CURRENT | MCL_FUTURE);
 #endif
 
+	stc_standalone_init();
+
 	cfg = xzalloc(sizeof(*cfg));
 	if (__stoken_read_rcfile(opt_rcfile, cfg,
 				 is_import ? &dbg : &warn) != ERR_NONE)
