@@ -36,6 +36,17 @@ struct stoken_ctx {
 	struct stoken_cfg	cfg;
 };
 
+static struct stoken_guid stoken_guid_list[] = {
+	{ "iphone",   "iPhone",        "556f1985-33dd-442c-9155-3a0e994f21b1" },
+	{ "android",  "Android",       "a01c4380-fc01-4df0-b113-7fb98ec74694" },
+	{ "bb",       "BlackBerry",    "868c28f8-31bf-4911-9876-ebece5c3f2ab" },
+	{ "bb10",     "BlackBerry 10", "b77a1d06-d505-4200-90d3-1bb397748704" },
+	{ "winphone", "Windows Phone", "c483b592-63f0-4f19-b4cb-a6bce8e57159" },
+	{ "win",      "Windows",       "8f94b226-d362-4204-ac52-3b21fa333b6f" },
+	{ "mac",      "Mac OSX",       "d0955a53-569b-4ecc-9cf7-6c2a59d4e775" },
+	{ },
+};
+
 /***********************************************************************
  * Internal functions (only called from within the stoken package)
  ***********************************************************************/
@@ -361,9 +372,13 @@ struct stoken_info *stoken_get_info(struct stoken_ctx *ctx)
 	struct stoken_info *info = calloc(1, sizeof(*info));
 	if (!info)
 		return NULL;
+
 	strncpy(info->serial, ctx->t->serial, sizeof(info->serial) - 1);
 	info->exp_date = securid_unix_exp_date(ctx->t);
 	info->interval = securid_token_interval(ctx->t);
+	info->token_version = ctx->t->version;
+	info->uses_pin = securid_pin_required(ctx->t);
+
 	return info;
 }
 
@@ -396,11 +411,16 @@ int stoken_check_pin(struct stoken_ctx *ctx, const char *pin)
 	return securid_pin_format_ok(pin) == ERR_NONE ? 0 : -EINVAL;
 }
 
+const struct stoken_guid *stoken_get_guid_list(void)
+{
+	return stoken_guid_list;
+}
+
 int stoken_check_devid(struct stoken_ctx *ctx, const char *devid)
 {
-	if (securid_decrypt_seed(ctx->t, "", devid) == ERR_BAD_DEVID)
-		return -EINVAL;
-	return 0;
+	if (securid_check_devid(ctx->t, devid) == ERR_NONE)
+		return 0;
+	return -EINVAL;
 }
 
 int stoken_decrypt_seed(struct stoken_ctx *ctx, const char *pass,
@@ -427,7 +447,8 @@ char *stoken_encrypt_seed(struct stoken_ctx *ctx, const char *pass,
 	ret = calloc(1, MAX_TOKEN_CHARS + 1);
 	if (!ret)
 		return NULL;
-	if (securid_encode_token(ctx->t, pass, devid, ret) != ERR_NONE) {
+
+	if (securid_encode_token(ctx->t, pass, devid, 2, ret) != ERR_NONE) {
 		free(ret);
 		return NULL;
 	}
@@ -437,11 +458,34 @@ char *stoken_encrypt_seed(struct stoken_ctx *ctx, const char *pass,
 int stoken_compute_tokencode(struct stoken_ctx *ctx, time_t when,
 	const char *pin, char *out)
 {
-	if (securid_pin_required(ctx->t) && pin) {
-		if (securid_pin_format_ok(pin) != ERR_NONE)
+	if (securid_pin_required(ctx->t)) {
+		if (pin && strlen(pin)) {
+			if (securid_pin_format_ok(pin) != ERR_NONE)
+				return -EINVAL;
+			strncpy(ctx->t->pin, pin, MAX_PIN + 1);
+		} else if (stoken_pin_required(ctx)) {
 			return -EINVAL;
-		strncpy(ctx->t->pin, pin, MAX_PIN + 1);
+		}
 	}
 	securid_compute_tokencode(ctx->t, when, out);
 	return 0;
 }
+
+char *stoken_format_tokencode(const char *tokencode)
+{
+	int code_len = strlen(tokencode);
+	char *str = malloc(code_len + 2);
+	int i, j;
+
+	if (!str)
+		return NULL;
+
+	for (i = 0, j = 0; i < code_len; i++) {
+		if (i == code_len / 2)
+			str[j++] = ' ';
+		str[j++] = tokencode[i];
+	}
+	str[j] = 0;
+
+	return str;
+}
