@@ -27,7 +27,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <tomcrypt.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <sys/types.h>
@@ -81,105 +80,30 @@ static uint8_t hex2byte(const char *in)
 	return (hex2nibble(in[0]) << 4) | hex2nibble(in[1]);
 }
 
-void aes128_ecb_encrypt(const uint8_t *key, const uint8_t *in, uint8_t *out)
-{
-	symmetric_key skey;
-	uint8_t tmp[AES_BLOCK_SIZE];
-
-	/* these shouldn't allocate memory or fail */
-	if (rijndael_setup(key, AES_KEY_SIZE, 0, &skey) != CRYPT_OK ||
-	    rijndael_ecb_encrypt(in, tmp, &skey) != CRYPT_OK)
-		abort();
-	rijndael_done(&skey);
-
-	/* in case "in" and "out" point to the same buffer */
-	memcpy(out, tmp, AES_BLOCK_SIZE);
-}
-
-void aes128_ecb_decrypt(const uint8_t *key, const uint8_t *in, uint8_t *out)
-{
-	symmetric_key skey;
-	uint8_t tmp[AES_BLOCK_SIZE];
-
-	if (rijndael_setup(key, AES_KEY_SIZE, 0, &skey) != CRYPT_OK ||
-	    rijndael_ecb_decrypt(in, tmp, &skey) != CRYPT_OK)
-		abort();
-	rijndael_done(&skey);
-
-	memcpy(out, tmp, AES_BLOCK_SIZE);
-}
-
-static void aes256_cbc_decrypt(const uint8_t *key, const uint8_t *in, int in_len,
-			       const uint8_t *iv, uint8_t *out)
-{
-	symmetric_key skey;
-	int i, j;
-	uint8_t local_iv[AES_BLOCK_SIZE];
-
-	rijndael_setup(key, AES256_KEY_SIZE, 0, &skey);
-
-	memcpy(local_iv, iv, AES_BLOCK_SIZE);
-	for (i = 0; i < in_len; i += AES_BLOCK_SIZE) {
-		rijndael_ecb_decrypt(in, out, &skey);
-		for (j = 0; j < AES_BLOCK_SIZE; j++)
-			out[j] ^= local_iv[j];
-		memcpy(local_iv, in, AES_BLOCK_SIZE);
-		in += AES_BLOCK_SIZE;
-		out += AES_BLOCK_SIZE;
-	}
-	rijndael_done(&skey);
-}
-
-static void aes256_cbc_encrypt(const uint8_t *key, const uint8_t *in, int in_len,
-			       const uint8_t *iv, uint8_t *out)
-{
-	symmetric_key skey;
-	int i, j;
-	uint8_t xored_in[AES_BLOCK_SIZE];
-
-	rijndael_setup(key, AES256_KEY_SIZE, 0, &skey);
-
-	for (i = 0; i < in_len; i += AES_BLOCK_SIZE) {
-		for (j = 0; j < AES_BLOCK_SIZE; j++) {
-			xored_in[j] = in[j] ^
-				      (i ? out[j - AES_BLOCK_SIZE] : iv[j]);
-		}
-		rijndael_ecb_encrypt(xored_in, out, &skey);
-		in += AES_BLOCK_SIZE;
-		out += AES_BLOCK_SIZE;
-	}
-	rijndael_done(&skey);
-}
-
 int securid_rand(void *out, int len, int paranoid)
 {
-	if (paranoid) {
-		/*
-		 * Use /dev/random for long lived key material but not for
-		 * test purposes.  This can block for a long time if entropy
-		 * is limited.
-		 */
-		int fd;
-		char *p = out;
+	int fd;
+	char *p = out;
 
-		fd = open("/dev/random", O_RDONLY);
-		if (fd < 0)
+	/*
+	 * Use /dev/random for long lived key material but not for
+	 * test purposes.  This can block for a long time if entropy
+	 * is limited.
+	 */
+	fd = open(paranoid ? "/dev/random" : "/dev/urandom", O_RDONLY);
+	if (fd < 0)
+		return ERR_GENERAL;
+
+	while (len) {
+		ssize_t ret = read(fd, p, len);
+		if (ret < 0) {
+			close(fd);
 			return ERR_GENERAL;
-
-		while (len) {
-			ssize_t ret = read(fd, p, len);
-			if (ret < 0) {
-				close(fd);
-				return ERR_GENERAL;
-			}
-			p += ret;
-			len -= ret;
 		}
-		close(fd);
-	} else {
-		if (rng_get_bytes(out, len, NULL) != len)
-			return ERR_GENERAL;
+		p += ret;
+		len -= ret;
 	}
+	close(fd);
 	return ERR_NONE;
 }
 
@@ -187,7 +111,7 @@ static void encrypt_then_xor(const uint8_t *key, uint8_t *work, uint8_t *enc)
 {
 	int i;
 
-	aes128_ecb_encrypt(key, work, enc);
+	stc_aes128_ecb_encrypt(key, work, enc);
 	for (i = 0; i < AES_BLOCK_SIZE; i++)
 		work[i] ^= enc[i];
 }
@@ -236,24 +160,15 @@ static uint16_t securid_shortmac(const uint8_t *in, int in_len)
 	return (hash[0] << 7) | (hash[1] >> 1);
 }
 
-static void sha256_hash(const uint8_t *in, int in_len, uint8_t *out)
-{
-	hash_state md;
-	sha256_init(&md);
-	sha256_process(&md, in, in_len);
-	sha256_done(&md, out);
-}
-
 static void sha256_hmac(const uint8_t *key, int key_len,
 			const uint8_t *msg, int msg_len, uint8_t *out)
 {
-	hash_state md;
 	uint8_t tmp_key[SHA256_HASH_SIZE], o_key_pad[SHA256_BLOCK_SIZE],
 		i_key_pad[SHA256_BLOCK_SIZE], inner_hash[SHA256_BLOCK_SIZE];
 	int i;
 
 	if (key_len > SHA256_BLOCK_SIZE) {
-		sha256_hash(key, key_len, tmp_key);
+		stc_sha256_hash(tmp_key, key, key_len, NULL);
 		key = tmp_key;
 		key_len = SHA256_HASH_SIZE;
 	}
@@ -265,15 +180,15 @@ static void sha256_hmac(const uint8_t *key, int key_len,
 		i_key_pad[i] ^= key[i];
 	}
 
-	sha256_init(&md);
-	sha256_process(&md, i_key_pad, SHA256_BLOCK_SIZE);
-	sha256_process(&md, msg, msg_len);
-	sha256_done(&md, inner_hash);
+	stc_sha256_hash(inner_hash,
+			i_key_pad, SHA256_BLOCK_SIZE,
+			msg, msg_len,
+			NULL);
 
-	sha256_init(&md);
-	sha256_process(&md, o_key_pad, SHA256_BLOCK_SIZE);
-	sha256_process(&md, inner_hash, SHA256_HASH_SIZE);
-	sha256_done(&md, out);
+	stc_sha256_hash(out,
+			o_key_pad, SHA256_BLOCK_SIZE,
+			inner_hash, SHA256_HASH_SIZE,
+			NULL);
 }
 
 static void sha256_pbkdf2(const uint8_t *pass, int pass_len,
@@ -473,22 +388,14 @@ static int v2_decrypt_seed(struct securid_token *t, const char *pass,
 	uint16_t device_id_hash;
 	int rc;
 
-	if (t->flags & FL_PASSPROT && !pass)
-		return ERR_MISSING_PASSWORD;
-	if (t->flags & FL_SNPROT && !devid)
-		return ERR_MISSING_PASSWORD;
-
-	rc = generate_key_hash(key_hash,
-			       t->flags & FL_PASSPROT ? pass : NULL,
-			       t->flags & FL_SNPROT ? devid : NULL,
-			       &device_id_hash, t);
+	rc = generate_key_hash(key_hash, pass, devid, &device_id_hash, t);
 	if (rc)
 		return rc;
 
 	if (t->flags & FL_SNPROT && device_id_hash != t->device_id_hash)
 		return ERR_BAD_DEVID;
 
-	aes128_ecb_decrypt(key_hash, t->enc_seed, t->dec_seed);
+	stc_aes128_ecb_decrypt(key_hash, t->enc_seed, t->dec_seed);
 	securid_mac(t->dec_seed, AES_KEY_SIZE, dec_seed_hash);
 	computed_mac = (dec_seed_hash[0] << 7) | (dec_seed_hash[1] >> 1);
 
@@ -538,7 +445,7 @@ static int v2_encode_token(struct securid_token *t, const char *pass,
 		return rc;
 
 	memset(d, 0, sizeof(d));
-	aes128_ecb_encrypt(key_hash, t->dec_seed, t->enc_seed);
+	stc_aes128_ecb_encrypt(key_hash, t->dec_seed, t->enc_seed);
 	memcpy(d, t->enc_seed, AES_KEY_SIZE);
 
 	set_bits(d, 128, 16, t->flags);
@@ -546,7 +453,7 @@ static int v2_encode_token(struct securid_token *t, const char *pass,
 	set_bits(d, 159, 15, securid_shortmac(t->dec_seed, AES_KEY_SIZE));
 	set_bits(d, 174, 15, t->device_id_hash);
 
-	sprintf(out, "%d%s", t->version, t->serial);
+	sprintf(out, "2%s", t->serial);
 	bits_to_numoutput(d, &out[BINENC_OFS], BINENC_BITS);
 
 	set_bits(d, 0, 15, securid_shortmac(out, CHECKSUM_OFS));
@@ -616,8 +523,7 @@ static int v3_decode_token(const char *in, struct securid_token *t)
 	if (!t->v3)
 		return ERR_NO_MEMORY;
 
-	if (base64_decode(decoded, strlen(decoded),
-			  (void *)t->v3, &actual) != CRYPT_OK ||
+	if (stc_b64_decode(decoded, strlen(decoded), (void *)t->v3, &actual) ||
 	    actual != sizeof(struct v3_token) ||
 	    t->v3->version != 0x03) {
 		free(t->v3);
@@ -644,7 +550,8 @@ static uint16_t v3_parse_date(uint8_t *in)
 		   ((uint64_t)in[3] <<  8) |
 		   ((uint64_t)in[4] <<  0);
 	longdate /= SECURID_V3_DAY;
-	return longdate - SECURID_EPOCH_DAYS;
+	longdate -= SECURID_EPOCH_DAYS;
+	return longdate <= SECURID_MAX_DATE ? longdate : SECURID_MAX_DATE;
 }
 
 static void v3_encode_date(uint8_t *out, uint16_t in)
@@ -675,7 +582,9 @@ static void v3_compute_hash(const char *pass, const char *devid,
 		pass_len = strlen(pass);
 		strncpy(&hash_buf[V3_NONCE_BYTES + V3_DEVID_CHARS], pass, MAX_PASS);
 	}
-	sha256_hash(hash_buf, V3_NONCE_BYTES + V3_DEVID_CHARS + pass_len, hash);
+	stc_sha256_hash(hash,
+			hash_buf, V3_NONCE_BYTES + V3_DEVID_CHARS + pass_len,
+			NULL);
 }
 
 static void v3_compute_hmac(struct v3_token *v3, const char *pass,
@@ -705,8 +614,6 @@ static int v3_decrypt_seed(struct securid_token *t,
 	uint8_t hash[SHA256_HASH_SIZE];
 	char devid[V3_DEVID_CHARS + 1];
 
-	if (pass && strlen(pass) > MAX_PASS)
-		return ERR_BAD_PASSWORD;
 	v3_scrub_devid(raw_devid, devid);
 
 	v3_compute_hash(NULL, devid, t->v3->nonce, hash);
@@ -722,10 +629,12 @@ static int v3_decrypt_seed(struct securid_token *t,
 		return ERR_CHECKSUM_FAILED;
 
 	v3_derive_key(pass, devid, t->v3->nonce, 1, hash);
-	aes256_cbc_decrypt(hash,
+	stc_aes256_cbc_decrypt(hash,
 			   t->v3->enc_payload, sizeof(struct v3_payload),
 			   t->v3->nonce, (void *)&payload);
 
+	if (strlen(payload.serial) != SERIAL_CHARS)
+		return ERR_GENERAL;
 	strncpy(t->serial, payload.serial, SERIAL_CHARS);
 	t->serial[SERIAL_CHARS] = 0;
 
@@ -779,14 +688,15 @@ static int v3_encode_token(struct securid_token *t, const char *pass,
 
 	v3_scrub_devid(raw_devid, devid);
 	v3_derive_key(pass, devid, v3.nonce, 1, key);
-	aes256_cbc_encrypt(key, (void *)&payload, sizeof(struct v3_payload),
+	stc_aes256_cbc_encrypt(key,
+			   (void *)&payload, sizeof(struct v3_payload),
 			   v3.nonce, v3.enc_payload);
 
 	v3_compute_hash(NULL, devid, v3.nonce, v3.nonce_devid_hash);
 	v3_compute_hash(pass, devid, v3.nonce, v3.nonce_devid_pass_hash);
 	v3_compute_hmac(&v3, pass, devid, v3.mac);
 
-	base64_encode((void *)&v3, sizeof(v3), raw_b64, &enclen);
+	stc_b64_encode((void *)&v3, sizeof(v3), raw_b64, &enclen);
 
 	for (i = 0; i < enclen; i++) {
 		char c = raw_b64[i];
@@ -824,6 +734,21 @@ int securid_decode_token(const char *in, struct securid_token *t)
 int securid_decrypt_seed(struct securid_token *t, const char *pass,
 			 const char *devid)
 {
+	if (t->flags & FL_PASSPROT) {
+		if (!pass || !strlen(pass))
+			return ERR_MISSING_PASSWORD;
+		if (strlen(pass) > MAX_PASS)
+			return ERR_BAD_PASSWORD;
+	} else
+		pass = NULL;
+
+	if (t->flags & FL_SNPROT) {
+		if (!devid || !strlen(devid))
+			return ERR_MISSING_PASSWORD;
+		/* NOTE: max length is checked elsewhere, as it varies */
+	} else
+		devid = NULL;
+
 	if (t->sdtid)
 		return sdtid_decrypt(t, pass);
 	else if (t->v3)
@@ -832,6 +757,15 @@ int securid_decrypt_seed(struct securid_token *t, const char *pass,
 		return v2_decrypt_seed(t, pass, devid);
 }
 
+int securid_check_devid(struct securid_token *t, const char *devid)
+{
+	int ret = securid_decrypt_seed(t, ".", devid);
+	if (ret == ERR_BAD_DEVID || ret == ERR_MISSING_PASSWORD)
+		return ERR_BAD_DEVID;
+	else
+		return ERR_NONE;
+}
+
 void securid_compute_tokencode(struct securid_token *t, time_t now,
 			       char *code_out)
 {
@@ -852,15 +786,15 @@ void securid_compute_tokencode(struct securid_token *t, time_t now,
 	bcd_time[6] = bcd_time[7] = 0;
 
 	key_from_time(bcd_time, 2, t->serial, key0);
-	aes128_ecb_encrypt(t->dec_seed, key0, key0);
+	stc_aes128_ecb_encrypt(t->dec_seed, key0, key0);
 	key_from_time(bcd_time, 3, t->serial, key1);
-	aes128_ecb_encrypt(key0, key1, key1);
+	stc_aes128_ecb_encrypt(key0, key1, key1);
 	key_from_time(bcd_time, 4, t->serial, key0);
-	aes128_ecb_encrypt(key1, key0, key0);
+	stc_aes128_ecb_encrypt(key1, key0, key0);
 	key_from_time(bcd_time, 5, t->serial, key1);
-	aes128_ecb_encrypt(key0, key1, key1);
+	stc_aes128_ecb_encrypt(key0, key1, key1);
 	key_from_time(bcd_time, 8, t->serial, key0);
-	aes128_ecb_encrypt(key1, key0, key0);
+	stc_aes128_ecb_encrypt(key1, key0, key0);
 
 	/* key0 now contains 4 consecutive token codes */
 	if (is_30)
@@ -885,7 +819,7 @@ void securid_compute_tokencode(struct securid_token *t, time_t now,
 }
 
 int securid_encode_token(const struct securid_token *t, const char *pass,
-			 const char *devid, char *out)
+			 const char *devid, int version, char *out)
 {
 	struct securid_token newt = *t;
 
@@ -902,7 +836,7 @@ int securid_encode_token(const struct securid_token *t, const char *pass,
 	} else
 		newt.flags |= FL_SNPROT;
 
-	if (t->version == 3)
+	if (version == 3)
 		return v3_encode_token(&newt, pass, devid, out);
 	else
 		return v2_encode_token(&newt, pass, devid, out);
@@ -923,7 +857,7 @@ int securid_random_token(struct securid_token *t)
 	t->dec_seed_hash = securid_shortmac(t->dec_seed, AES_KEY_SIZE);
 
 	generate_key_hash(key_hash, NULL, NULL, &t->device_id_hash, t);
-	aes128_ecb_encrypt(key_hash, t->dec_seed, t->enc_seed);
+	stc_aes128_ecb_encrypt(key_hash, t->dec_seed, t->enc_seed);
 	t->has_enc_seed = 1;
 
 	t->version = 2;
@@ -949,7 +883,10 @@ time_t securid_unix_exp_date(const struct securid_token *t)
 	 * expiration checks.
 	 */
 	if (t->version == 3 && !t->exp_date)
-		return 0x7fffffff;
+		return MAX_TIME_T;
+	if (t->exp_date > SECURID_MAX_DATE)
+		return MAX_TIME_T;
+
 	return SECURID_EPOCH + (t->exp_date + 1) * 60 * 60 * 24;
 }
 
@@ -1083,7 +1020,7 @@ char *securid_encrypt_pin(const char *pin, const char *password)
 
 	for (i = 0; i < AES_BLOCK_SIZE; i++)
 		buf[i] ^= iv[i];
-	aes128_ecb_encrypt(passhash, buf, buf);
+	stc_aes128_ecb_encrypt(passhash, buf, buf);
 
 	ret = malloc(AES_BLOCK_SIZE * 2 * 2 + 1);
 	if (!ret)
@@ -1112,7 +1049,7 @@ int securid_decrypt_pin(const char *enc_pin, const char *password, char *pin)
 	}
 
 	securid_mac(password, strlen(password), passhash);
-	aes128_ecb_decrypt(passhash, buf, buf);
+	stc_aes128_ecb_decrypt(passhash, buf, buf);
 
 	for (i = 0; i < AES_BLOCK_SIZE; i++)
 		buf[i] ^= iv[i];
