@@ -30,7 +30,6 @@
 
 #define AES_BLOCK_SIZE		16
 #define AES_KEY_SIZE		16
-#define AES256_KEY_SIZE		32
 
 #define SHA256_BLOCK_SIZE	64
 #define SHA256_HASH_SIZE	32
@@ -105,6 +104,11 @@
 /* V3 tokens use 1970/01/01 as the epoch, but each day has 337500 ticks */
 #define SECURID_V3_DAY		337500
 
+/* Avoid 32-bit time_t overflows (January 2038) */
+#define MAX_TIME_T		0x7fffffff
+#define SECURID_MAX_SECS	(MAX_TIME_T - SECURID_EPOCH)
+#define SECURID_MAX_DATE	(SECURID_MAX_SECS / (24*60*60) - 1)
+
 struct sdtid;
 struct v3_token;
 
@@ -133,30 +137,44 @@ struct securid_token {
 	struct v3_token		*v3;
 };
 
-int securid_decode_token(const char *in, struct securid_token *t);
-int securid_decrypt_seed(struct securid_token *t, const char *pass,
-	const char *devid);
-void securid_compute_tokencode(struct securid_token *t, time_t now,
-	char *code_out);
-void securid_token_info(const struct securid_token *t,
+STOKEN_EXPORT int securid_decode_token(const char *in,
+				       struct securid_token *t);
+
+STOKEN_EXPORT int securid_decrypt_seed(struct securid_token *t,
+				       const char *pass,
+				       const char *devid);
+
+STOKEN_EXPORT int securid_check_devid(struct securid_token *t,
+				      const char *devid);
+
+STOKEN_EXPORT void securid_compute_tokencode(struct securid_token *t,
+					     time_t now,
+					     char *code_out);
+
+STOKEN_EXPORT void securid_token_info(const struct securid_token *t,
 	void (*callback)(const char *key, const char *value));
-int securid_encode_token(const struct securid_token *t, const char *pass,
-	const char *devid, char *out);
-int securid_random_token(struct securid_token *t);
-int securid_check_exp(struct securid_token *t, time_t now);
-time_t securid_unix_exp_date(const struct securid_token *t);
-int securid_token_interval(const struct securid_token *t);
 
-char *securid_encrypt_pin(const char *pin, const char *password);
-int securid_decrypt_pin(const char *enc_pin, const char *password, char *pin);
+STOKEN_EXPORT int securid_encode_token(const struct securid_token *t,
+				       const char *pass,
+				       const char *devid,
+				       int version,
+				       char *out);
 
-int securid_pin_format_ok(const char *pin);
-int securid_pin_required(const struct securid_token *t);
-int securid_pass_required(const struct securid_token *t);
-int securid_devid_required(const struct securid_token *t);
+STOKEN_EXPORT int securid_random_token(struct securid_token *t);
+STOKEN_EXPORT int securid_check_exp(struct securid_token *t, time_t now);
+STOKEN_EXPORT time_t securid_unix_exp_date(const struct securid_token *t);
+STOKEN_EXPORT int securid_token_interval(const struct securid_token *t);
 
-void aes128_ecb_decrypt(const uint8_t *key, const uint8_t *in, uint8_t *out);
-void aes128_ecb_encrypt(const uint8_t *key, const uint8_t *in, uint8_t *out);
-int securid_rand(void *out, int len, int paranoid);
+STOKEN_EXPORT char *securid_encrypt_pin(const char *pin, const char *password);
+
+STOKEN_EXPORT int securid_decrypt_pin(const char *enc_pin,
+				      const char *password,
+				      char *pin);
+
+STOKEN_EXPORT int securid_pin_format_ok(const char *pin);
+STOKEN_EXPORT int securid_pin_required(const struct securid_token *t);
+STOKEN_EXPORT int securid_pass_required(const struct securid_token *t);
+STOKEN_EXPORT int securid_devid_required(const struct securid_token *t);
+STOKEN_EXPORT int securid_rand(void *out, int len, int paranoid);
 
 #endif /* !__STOKEN_SECURID_H__ */
